@@ -249,4 +249,79 @@ static __always_inline __u64 get_sock_ident(struct sock *sk)
 	return (__u64)sk;
 }
 
+/**
+ * During kernel 6.6 development cycle, several bitfields in struct inet_sock gone,
+ * they are placed in inet_sock::inet_flags instead ([0]).
+ *
+ * References:
+ *   [0]: https://lore.kernel.org/all/20230816081547.1272409-1-edumazet@google.com/
+ */
+struct inet_sock___o {
+	__u8 freebind: 1;
+	__u8 transparent: 1;
+	__u8 bind_address_no_port: 1;
+};
+
+enum {
+	INET_FLAGS_FREEBIND___x = 11,
+	INET_FLAGS_TRANSPARENT___x = 15,
+	INET_FLAGS_BIND_ADDRESS_NO_PORT___x = 18,
+};
+
+struct inet_sock___x {
+	unsigned long inet_flags;
+};
+
+static __always_inline __u8 get_inet_sock_freebind(void *inet_sock)
+{
+	unsigned long inet_flags;
+
+	if (bpf_core_field_exists(struct inet_sock___o, freebind))
+		return BPF_CORE_READ_BITFIELD_PROBED((struct inet_sock___o *)inet_sock, freebind);
+
+	inet_flags = BPF_CORE_READ((struct inet_sock___x *)inet_sock, inet_flags);
+	return (1 << INET_FLAGS_FREEBIND___x) & inet_flags ? 1 : 0;
+}
+
+static __always_inline __u8 get_inet_sock_transparent(void *inet_sock)
+{
+	unsigned long inet_flags;
+
+	if (bpf_core_field_exists(struct inet_sock___o, transparent))
+		return BPF_CORE_READ_BITFIELD_PROBED((struct inet_sock___o *)inet_sock, transparent);
+
+	inet_flags = BPF_CORE_READ((struct inet_sock___x *)inet_sock, inet_flags);
+	return (1 << INET_FLAGS_TRANSPARENT___x) & inet_flags ? 1 : 0;
+}
+
+static __always_inline __u8 get_inet_sock_bind_address_no_port(void *inet_sock)
+{
+	unsigned long inet_flags;
+
+	if (bpf_core_field_exists(struct inet_sock___o, bind_address_no_port))
+		return BPF_CORE_READ_BITFIELD_PROBED((struct inet_sock___o *)inet_sock, bind_address_no_port);
+
+	inet_flags = BPF_CORE_READ((struct inet_sock___x *)inet_sock, inet_flags);
+	return (1 << INET_FLAGS_BIND_ADDRESS_NO_PORT___x) & inet_flags ? 1 : 0;
+}
+
+/**
+ * commit 736c55a02c47 ("sched/fair: Rename cfs_rq.nr_running into nr_queued")
+ * renamed cfs_rq::nr_running to cfs_rq::nr_queued.
+ *
+ * References:
+ *   [0]: https://github.com/torvalds/linux/commit/736c55a02c47
+ */
+struct cfs_rq___pre_v614 {
+	unsigned int		nr_running;
+};
+
+static __always_inline __u8 cfs_rq_get_nr_running_or_nr_queued(void *cfs_rq)
+{
+	if (bpf_core_field_exists(struct cfs_rq___pre_v614, nr_running))
+		return BPF_CORE_READ((struct cfs_rq___pre_v614 *)cfs_rq, nr_running);
+
+	return BPF_CORE_READ((struct cfs_rq *)cfs_rq, nr_queued);
+}
+
 #endif /* __CORE_FIXES_BPF_H */
