@@ -40,6 +40,8 @@ enum fs_type {
 	NFS,
 	XFS,
 	F2FS,
+	BCACHEFS,
+	ZFS,
 };
 
 static struct fs_config {
@@ -76,6 +78,18 @@ static struct fs_config {
 		[F_OPEN] = "f2fs_file_open",
 		[F_FSYNC] = "f2fs_sync_file",
 	}},
+	[BCACHEFS] = { "bcachefs", {
+		[F_READ] = "bch2_read_iter",
+		[F_WRITE] = "bch2_write_iter",
+		[F_OPEN] = "bch2_open",
+		[F_FSYNC] = "bch2_fsync",
+	}},
+	[ZFS] = { "zfs", {
+		[F_READ] = "zpl_iter_read",
+		[F_WRITE] = "zpl_iter_write",
+		[F_OPEN] = "zpl_open",
+		[F_FSYNC] = "zpl_fsync",
+	}},
 };
 
 static char file_op[] = {
@@ -109,13 +123,13 @@ const char argp_program_doc[] =
 "    fsslower -t xfs -c -d 1      # trace xfs operations for 1s with csv output\n";
 
 static const struct argp_option opts[] = {
-	{ "csv", 'c', NULL, 0, "Output as csv" },
-	{ "duration", 'd', "DURATION", 0, "Total duration of trace in seconds" },
-	{ "pid", 'p', "PID", 0, "Process ID to trace" },
-	{ "min", 'm', "MIN", 0, "Min latency to trace, in ms (default 10)" },
-	{ "type", 't', "Filesystem", 0, "Which filesystem to trace, [btrfs/ext4/nfs/xfs/f2fs]" },
-	{ "verbose", 'v', NULL, 0, "Verbose debug output" },
-	{ NULL, 'h', NULL, OPTION_HIDDEN, "Show the full help" },
+	{ "csv", 'c', NULL, 0, "Output as csv", 0 },
+	{ "duration", 'd', "DURATION", 0, "Total duration of trace in seconds", 0 },
+	{ "pid", 'p', "PID", 0, "Process ID to trace", 0 },
+	{ "min", 'm', "MIN", 0, "Min latency to trace, in ms (default 10)", 0 },
+	{ "type", 't', "Filesystem", 0, "Which filesystem to trace, [btrfs/ext4/nfs/xfs/f2fs/bcachefs/zfs]", 0 },
+	{ "verbose", 'v', NULL, 0, "Verbose debug output", 0 },
+	{ NULL, 'h', NULL, OPTION_HIDDEN, "Show the full help", 0 },
 	{},
 };
 
@@ -154,6 +168,10 @@ static error_t parse_arg(int key, char *arg, struct argp_state *state)
 			fs_type = XFS;
 		} else if (!strcmp(arg, "f2fs")) {
 			fs_type = F2FS;
+		} else if (!strcmp(arg, "bcachefs")) {
+			fs_type = BCACHEFS;
+		} else if (!strcmp(arg, "zfs")) {
+			fs_type = ZFS;
 		} else {
 			warn("invalid filesystem\n");
 			argp_usage(state);
@@ -190,6 +208,10 @@ static void alias_parse(char *prog)
 		fs_type = XFS;
 	} else if (strstr(name, "f2fsslower")){
 		fs_type = F2FS;
+	} else if (strstr(name, "bcachefsslower")){
+		fs_type = BCACHEFS;
+	} else if (!strcmp(name, "zfsslower")) {
+		fs_type = ZFS;
 	}
 }
 
