@@ -13,15 +13,15 @@ const volatile bool targ_per_thread = false;
 struct {
 	__uint(type, BPF_MAP_TYPE_HASH);
 	__uint(max_entries, MAX_ENTRIES);
-	__type(key, struct key_info);
-	__type(value, struct value_info);
+	__type(key, struct llcstat_key_info);
+	__type(value, struct llcstat_value_info);
 } infos SEC(".maps");
 
 static __always_inline
 int trace_event(__u64 sample_period, bool miss)
 {
-	struct key_info key = {};
-	struct value_info *infop, zero = {};
+	struct llcstat_key_info key = {};
+	struct llcstat_value_info *infop, zero = {};
 
 	u64 pid_tgid = bpf_get_current_pid_tgid();
 	key.cpu = bpf_get_smp_processor_id();
