@@ -14,7 +14,7 @@ struct {
 __u64 latency = 0;
 __u64 num = 0;
 
-static int __migrate_misplaced_page(void)
+static int __migrate_misplaced(void)
 {
 	u32 pid = bpf_get_current_pid_tgid();
 	u64 ts = bpf_ktime_get_ns();
@@ -26,16 +26,28 @@ static int __migrate_misplaced_page(void)
 SEC("fentry/migrate_misplaced_page")
 int BPF_PROG(fentry_migrate_misplaced_page)
 {
-	return __migrate_misplaced_page();
+	return __migrate_misplaced();
+}
+
+SEC("fentry/migrate_misplaced_folio")
+int BPF_PROG(fentry_migrate_misplaced_folio)
+{
+	return __migrate_misplaced();
 }
 
 SEC("kprobe/migrate_misplaced_page")
 int BPF_PROG(kprobe_migrate_misplaced_page)
 {
-	return __migrate_misplaced_page();
+	return __migrate_misplaced();
 }
 
-static int __migrate_misplaced_page_exit(void)
+SEC("kprobe/migrate_misplaced_folio")
+int BPF_PROG(kprobe_migrate_misplaced_folio)
+{
+	return __migrate_misplaced();
+}
+
+static int __migrate_misplaced_exit(void)
 {
 	u32 pid = bpf_get_current_pid_tgid();
 	u64 *tsp, ts = bpf_ktime_get_ns();
@@ -58,13 +70,25 @@ cleanup:
 SEC("fexit/migrate_misplaced_page")
 int BPF_PROG(fexit_migrate_misplaced_page_exit)
 {
-	return __migrate_misplaced_page_exit();
+	return __migrate_misplaced_exit();
+}
+
+SEC("fexit/migrate_misplaced_folio")
+int BPF_PROG(fexit_migrate_misplaced_folio_exit)
+{
+	return __migrate_misplaced_exit();
 }
 
 SEC("kretprobe/migrate_misplaced_page")
 int BPF_PROG(kretprobe_migrate_misplaced_page_exit)
 {
-	return __migrate_misplaced_page_exit();
+	return __migrate_misplaced_exit();
+}
+
+SEC("kretprobe/migrate_misplaced_folio")
+int BPF_PROG(kretprobe_migrate_misplaced_folio_exit)
+{
+	return __migrate_misplaced_exit();
 }
 
 char LICENSE[] SEC("license") = "GPL";
