@@ -31,13 +31,13 @@ const char argp_program_doc[] =
 "USAGE: readahead [--help] [-d DURATION]\n"
 "\n"
 "EXAMPLES:\n"
-"    readahead              # summarize on-CPU time as a histogram"
+"    readahead              # summarize on-CPU time as a histogram\n"
 "    readahead -d 10        # trace for 10 seconds only\n";
 
 static const struct argp_option opts[] = {
-	{ "duration", 'd', "DURATION", 0, "Duration to trace"},
-	{ "verbose", 'v', NULL, 0, "Verbose debug output" },
-	{ NULL, 'h', NULL, OPTION_HIDDEN, "Show the full help" },
+	{ "duration", 'd', "DURATION", 0, "Duration to trace", 0 },
+	{ "verbose", 'v', NULL, 0, "Verbose debug output", 0 },
+	{ NULL, 'h', NULL, OPTION_HIDDEN, "Show the full help", 0 },
 	{},
 };
 
@@ -64,8 +64,7 @@ static error_t parse_arg(int key, char *arg, struct argp_state *state)
 	return 0;
 }
 
-int libbpf_print_fn(enum libbpf_print_level level,
-		    const char *format, va_list args)
+static int libbpf_print_fn(enum libbpf_print_level level, const char *format, va_list args)
 {
 	if (level == LIBBPF_DEBUG && !env.verbose)
 		return 0;
@@ -81,6 +80,18 @@ static int readahead__set_attach_target(struct bpf_program *prog)
 {
 	int err;
 
+	/*
+	 * 56a4d67c264e ("mm/readahead: Switch to page_cache_ra_order") in v5.18
+	 * renamed do_page_cache_ra to page_cache_ra_order
+	 */
+	err = bpf_program__set_attach_target(prog, 0, "page_cache_ra_order");
+	if (!err)
+		return 0;
+
+	/*
+	 * 8238287eadb2 ("mm/readahead: make do_page_cache_ra take a readahead_control")
+	 * in v5.10 renamed __do_page_cache_readahead to do_page_cache_ra
+	*/
 	err = bpf_program__set_attach_target(prog, 0, "do_page_cache_ra");
 	if (!err)
 		return 0;
@@ -95,6 +106,52 @@ static int readahead__set_attach_target(struct bpf_program *prog)
 	return err;
 }
 
+static int attach_access(struct readahead_bpf *obj)
+{
+	bpf_program__set_autoload(obj->progs.folio_mark_accessed, false);
+	bpf_program__set_autoload(obj->progs.mark_page_accessed, false);
+
+	/*
+	 * 76580b65 ("mm/swap: Add folio_mark_accessed()") in v5.15
+	 * convert mark_page_accessed() to folio_mark_accessed().
+	 */
+	if (fentry_can_attach("folio_mark_accessed", NULL))
+		return bpf_program__set_autoload(obj->progs.folio_mark_accessed, true);
+
+	if (fentry_can_attach("mark_page_accessed", NULL))
+		return bpf_program__set_autoload(obj->progs.mark_page_accessed, true);
+	
+	fprintf(stderr, "failed to attach to access functions\n");
+	return -1;
+}
+
+static int attach_alloc_ret(struct readahead_bpf *obj)
+{
+	bpf_program__set_autoload(obj->progs.page_cache_alloc_ret, false);
+	bpf_program__set_autoload(obj->progs.filemap_alloc_folio_ret, false);
+	bpf_program__set_autoload(obj->progs.filemap_alloc_folio_noprof_ret, false);
+
+	/*
+	 * b951aaff5035 ("mm: enable page allocation tagging") in v6.10
+	 * renamed filemap_alloc_folio to filemap_alloc_folio_noprof
+	 */
+	if (fentry_can_attach("filemap_alloc_folio_noprof", NULL))
+		return bpf_program__set_autoload(obj->progs.filemap_alloc_folio_noprof_ret, true);
+
+	/*
+	 * bb3c579e25e5 ("mm/filemap: Add filemap_alloc_folio") in v5.16
+	 * changed __page_cache_alloc to be a wrapper of filemap_alloc_folio
+	 */
+	if (fentry_can_attach("filemap_alloc_folio", NULL))
+		return bpf_program__set_autoload(obj->progs.filemap_alloc_folio_ret, true);
+
+	if (fentry_can_attach("__page_cache_alloc", NULL))
+		return bpf_program__set_autoload(obj->progs.page_cache_alloc_ret, true);
+
+	fprintf(stderr, "failed to attach to alloc functions\n");
+	return -1;
+}
+
 int main(int argc, char **argv)
 {
 	static const struct argp argp = {
@@ -112,22 +169,18 @@ int main(int argc, char **argv)
 
 	libbpf_set_print(libbpf_print_fn);
 
-	err = bump_memlock_rlimit();
-	if (err) {
-		fprintf(stderr, "failed to increase rlimit: %d\n", err);
-		return 1;
-	}
-
 	obj = readahead_bpf__open();
 	if (!obj) {
 		fprintf(stderr, "failed to open BPF object\n");
 		return 1;
 	}
 
-	/*
-	 * Starting from v5.10-rc1 (8238287), __do_page_cache_readahead has
-	 * renamed to do_page_cache_ra. So we specify the function dynamically.
-	 */
+	err = attach_access(obj);
+	if (err)
+		goto cleanup;
+	err = attach_alloc_ret(obj);
+	if (err)
+		goto cleanup;
 	err = readahead__set_attach_target(obj->progs.do_page_cache_ra);
 	if (err)
 		goto cleanup;
