@@ -55,15 +55,15 @@ const char argp_program_doc[] =
 #define OPT_PIDNSS	1	/* --pidnss */
 
 static const struct argp_option opts[] = {
-	{ "timestamp", 'T', NULL, 0, "Include timestamp on output" },
-	{ "milliseconds", 'm', NULL, 0, "Millisecond histogram" },
-	{ "pidnss", OPT_PIDNSS, NULL, 0, "Print a histogram per PID namespace" },
-	{ "pids", 'P', NULL, 0, "Print a histogram per process ID" },
-	{ "tids", 'L', NULL, 0, "Print a histogram per thread ID" },
-	{ "pid", 'p', "PID", 0, "Trace this PID only" },
-	{ "verbose", 'v', NULL, 0, "Verbose debug output" },
-	{ "cgroup", 'c', "/sys/fs/cgroup/unified", 0, "Trace process in cgroup path"},
-	{ NULL, 'h', NULL, OPTION_HIDDEN, "Show the full help" },
+	{ "timestamp", 'T', NULL, 0, "Include timestamp on output", 0 },
+	{ "milliseconds", 'm', NULL, 0, "Millisecond histogram", 0 },
+	{ "pidnss", OPT_PIDNSS, NULL, 0, "Print a histogram per PID namespace", 0 },
+	{ "pids", 'P', NULL, 0, "Print a histogram per process ID", 0 },
+	{ "tids", 'L', NULL, 0, "Print a histogram per thread ID", 0 },
+	{ "pid", 'p', "PID", 0, "Trace this PID only", 0 },
+	{ "verbose", 'v', NULL, 0, "Verbose debug output", 0 },
+	{ "cgroup", 'c', "/sys/fs/cgroup/unified", 0, "Trace process in cgroup path", 0 },
+	{ NULL, 'h', NULL, OPTION_HIDDEN, "Show the full help", 0 },
 	{},
 };
 
@@ -204,7 +204,6 @@ int main(int argc, char **argv)
 		return 1;
 	}
 
-	libbpf_set_strict_mode(LIBBPF_STRICT_ALL);
 	libbpf_set_print(libbpf_print_fn);
 
 	obj = runqlat_bpf__open();
