@@ -92,7 +92,7 @@ std::string sanitize_str(std::string str, bool (*validator)(char),
 StatusTuple BPF::init_usdt(const USDT& usdt) {
   USDT u(usdt);
   StatusTuple init_stp = u.init();
-  if (init_stp.code() != 0) {
+  if (!init_stp.ok()) {
     return init_stp;
   }
 
@@ -112,19 +112,20 @@ StatusTuple BPF::init(const std::string& bpf_program,
   usdt_.reserve(usdt.size());
   for (const auto& u : usdt) {
     StatusTuple init_stp = init_usdt(u);
-    if (init_stp.code() != 0) {
+    if (!init_stp.ok()) {
       init_fail_reset();
       return init_stp;
     }
   }
 
-  auto flags_len = cflags.size();
-  const char* flags[flags_len];
-  for (size_t i = 0; i < flags_len; i++)
-    flags[i] = cflags[i].c_str();
+  std::vector<const char*> flags;
+  for (const auto& c: cflags)
+    flags.push_back(c.c_str());
 
   all_bpf_program_ += bpf_program;
-  if (bpf_module_->load_string(all_bpf_program_, flags, flags_len) != 0) {
+  if (bpf_module_->load_string(all_bpf_program_,
+                               flags.data(),
+                               flags.size()) != 0) {
     init_fail_reset();
     return StatusTuple(-1, "Unable to initialize BPF program");
   }
@@ -134,7 +135,7 @@ StatusTuple BPF::init(const std::string& bpf_program,
 
 BPF::~BPF() {
   auto res = detach_all();
-  if (res.code() != 0)
+  if (!res.ok())
     std::cerr << "Failed to detach all probes on destruction: " << std::endl
               << res.msg() << std::endl;
   bcc_free_buildsymcache(bsymcache_);
@@ -147,7 +148,7 @@ StatusTuple BPF::detach_all() {
 
   for (auto& it : kprobes_) {
     auto res = detach_kprobe_event(it.first, it.second);
-    if (res.code() != 0) {
+    if (!res.ok()) {
       error_msg += "Failed to detach kprobe event " + it.first + ": ";
       error_msg += res.msg() + "\n";
       has_error = true;
@@ -156,7 +157,7 @@ StatusTuple BPF::detach_all() {
 
   for (auto& it : uprobes_) {
     auto res = detach_uprobe_event(it.first, it.second);
-    if (res.code() != 0) {
+    if (!res.ok()) {
       error_msg += "Failed to detach uprobe event " + it.first + ": ";
       error_msg += res.msg() + "\n";
       has_error = true;
@@ -165,7 +166,7 @@ StatusTuple BPF::detach_all() {
 
   for (auto& it : tracepoints_) {
     auto res = detach_tracepoint_event(it.first, it.second);
-    if (res.code() != 0) {
+    if (!res.ok()) {
       error_msg += "Failed to detach Tracepoint " + it.first + ": ";
       error_msg += res.msg() + "\n";
       has_error = true;
@@ -174,7 +175,7 @@ StatusTuple BPF::detach_all() {
 
   for (auto& it : raw_tracepoints_) {
     auto res = detach_raw_tracepoint_event(it.first, it.second);
-    if (res.code() != 0) {
+    if (!res.ok()) {
       error_msg += "Failed to detach Raw tracepoint " + it.first + ": ";
       error_msg += res.msg() + "\n";
       has_error = true;
@@ -183,7 +184,7 @@ StatusTuple BPF::detach_all() {
 
   for (auto& it : perf_buffers_) {
     auto res = it.second->close_all_cpu();
-    if (res.code() != 0) {
+    if (!res.ok()) {
       error_msg += "Failed to close perf buffer " + it.first + ": ";
       error_msg += res.msg() + "\n";
       has_error = true;
@@ -193,7 +194,7 @@ StatusTuple BPF::detach_all() {
 
   for (auto& it : perf_event_arrays_) {
     auto res = it.second->close_all_cpu();
-    if (res.code() != 0) {
+    if (!res.ok()) {
       error_msg += "Failed to close perf event array " + it.first + ": ";
       error_msg += res.msg() + "\n";
       has_error = true;
@@ -203,7 +204,7 @@ StatusTuple BPF::detach_all() {
 
   for (auto& it : perf_events_) {
     auto res = detach_perf_event_all_cpu(it.second);
-    if (res.code() != 0) {
+    if (!res.ok()) {
       error_msg += res.msg() + "\n";
       has_error = true;
     }
@@ -303,7 +304,7 @@ StatusTuple BPF::attach_uprobe(const std::string& binary_path,
 StatusTuple BPF::attach_usdt_without_validation(const USDT& u, pid_t pid) {
   auto& probe = *static_cast<::USDT::Probe*>(u.probe_.get());
   if (!uprobe_ref_ctr_supported() && !probe.enable(u.probe_func_))
-    return StatusTuple(-1, "Unable to enable USDT %s" + u.print_name());
+    return StatusTuple(-1, "Unable to enable USDT %s", u.print_name().c_str());
 
   bool failed = false;
   std::string err_msg;
@@ -610,7 +611,7 @@ StatusTuple BPF::detach_perf_event_raw(void* perf_event_attr) {
 }
 
 StatusTuple BPF::open_perf_event(const std::string& name, uint32_t type,
-                                 uint64_t config) {
+                                 uint64_t config, int pid) {
   if (perf_event_arrays_.find(name) == perf_event_arrays_.end()) {
     TableStorage::iterator it;
     if (!bpf_module_->table_storage().Find(Path({bpf_module_->id(), name}), it))
@@ -619,7 +620,7 @@ StatusTuple BPF::open_perf_event(const std::string& name, uint32_t type,
     perf_event_arrays_[name] = new BPFPerfEventArray(it->second);
   }
   auto table = perf_event_arrays_[name];
-  TRY2(table->open_all_cpu(type, config));
+  TRY2(table->open_all_cpu(type, config, pid));
   return StatusTuple::OK();
 }
 
@@ -671,7 +672,7 @@ int BPF::poll_perf_buffer(const std::string& name, int timeout_ms) {
 }
 
 StatusTuple BPF::load_func(const std::string& func_name, bpf_prog_type type,
-                           int& fd) {
+                           int& fd, unsigned flags, bpf_attach_type expected_attach_type) {
   if (funcs_.find(func_name) != funcs_.end()) {
     fd = funcs_[func_name];
     return StatusTuple::OK();
@@ -692,7 +693,7 @@ StatusTuple BPF::load_func(const std::string& func_name, bpf_prog_type type,
   fd = bpf_module_->bcc_func_load(type, func_name.c_str(),
                      reinterpret_cast<struct bpf_insn*>(func_start), func_size,
                      bpf_module_->license(), bpf_module_->kern_version(),
-                     log_level, nullptr, 0);
+                     log_level, nullptr, 0, nullptr, flags, expected_attach_type);
 
   if (fd < 0)
     return StatusTuple(-1, "Failed to load %s: %d", func_name.c_str(), fd);
@@ -833,13 +834,6 @@ BPFStackBuildIdTable BPF::get_stackbuildid_table(const std::string &name, bool u
   return BPFStackBuildIdTable({}, use_debug_file, check_debug_file_crc, get_bsymcache());
 }
 
-BPFMapInMapTable BPF::get_map_in_map_table(const std::string& name) {
-  TableStorage::iterator it;
-  if (bpf_module_->table_storage().Find(Path({bpf_module_->id(), name}), it))
-    return BPFMapInMapTable(it->second);
-  return BPFMapInMapTable({});
-}
-
 BPFSockmapTable BPF::get_sockmap_table(const std::string& name) {
   TableStorage::iterator it;
   if (bpf_module_->table_storage().Find(Path({bpf_module_->id(), name}), it))
@@ -860,6 +854,17 @@ bool BPF::add_module(std::string module)
     false : true;
 }
 
+namespace {
+
+constexpr size_t kEventNameSizeLimit = 224;
+
+std::string shorten_event_name(const std::string& name) {
+  std::string hash = uint_to_hex(std::hash<std::string>{}(name));
+  return name.substr(0, kEventNameSizeLimit - hash.size()) + hash;
+}
+
+} // namespace
+
 std::string BPF::get_uprobe_event(const std::string& binary_path,
                                   uint64_t offset, bpf_probe_attach_type type,
                                   pid_t pid) {
@@ -868,6 +873,9 @@ std::string BPF::get_uprobe_event(const std::string& binary_path,
   res += "_0x" + uint_to_hex(offset);
   if (pid != -1)
     res += "_" + std::to_string(pid);
+  if (res.size() > kEventNameSizeLimit) {
+    return shorten_event_name(res);
+  }
   return res;
 }
 
