@@ -19,7 +19,9 @@
 #include <tuple>
 #include <vector>
 
-#if LLVM_MAJOR_VERSION >= 15
+#include "bcc_debug.h"
+
+#if LLVM_VERSION_MAJOR >= 15
 #include <llvm/DebugInfo/DWARF/DWARFCompileUnit.h>
 #endif
 #include <llvm/DebugInfo/DWARF/DWARFContext.h>
@@ -32,17 +34,15 @@
 #include <llvm/MC/MCInstrInfo.h>
 #include <llvm/MC/MCObjectFileInfo.h>
 #include <llvm/MC/MCRegisterInfo.h>
-#if LLVM_MAJOR_VERSION >= 15
+#if LLVM_VERSION_MAJOR >= 15
 #include <llvm/MC/MCSubtargetInfo.h>
 #endif
-#if LLVM_MAJOR_VERSION >= 14
+#if LLVM_VERSION_MAJOR >= 14
 #include <llvm/MC/TargetRegistry.h>
 #else
 #include <llvm/Support/TargetRegistry.h>
 #endif
 
-#include "bcc_debug.h"
-
 namespace ebpf {
 
 // ld_pseudo can only be disassembled properly
@@ -114,7 +114,11 @@ void SourceDebugger::getDebugSections(
 
 void SourceDebugger::dump() {
   string Error;
+#if LLVM_VERSION_MAJOR >= 21
+  string TripleStr(mod_->getTargetTriple().str());
+#else
   string TripleStr(mod_->getTargetTriple());
+#endif
   Triple TheTriple(TripleStr);
   const Target *T = TargetRegistry::lookupTarget(TripleStr, Error);
   if (!T) {
@@ -127,12 +131,9 @@ void SourceDebugger::dump() {
     errs() << "Debug Error: cannot get register info\n";
     return;
   }
-#if LLVM_MAJOR_VERSION >= 10
+
   MCTargetOptions MCOptions;
   std::unique_ptr<MCAsmInfo> MAI(T->createMCAsmInfo(*MRI, TripleStr, MCOptions));
-#else
-  std::unique_ptr<MCAsmInfo> MAI(T->createMCAsmInfo(*MRI, TripleStr));
-#endif
   if (!MAI) {
     errs() << "Debug Error: cannot get assembly info\n";
     return;
@@ -141,7 +142,7 @@ void SourceDebugger::dump() {
   std::unique_ptr<MCSubtargetInfo> STI(
       T->createMCSubtargetInfo(TripleStr, "", ""));
   MCObjectFileInfo MOFI;
-#if LLVM_MAJOR_VERSION >= 13
+#if LLVM_VERSION_MAJOR >= 13
   MCContext Ctx(TheTriple, MAI.get(), MRI.get(), STI.get(), nullptr);
   Ctx.setObjectFileInfo(&MOFI);
   MOFI.initMCObjectFileInfo(Ctx, false, false);
@@ -174,13 +175,7 @@ void SourceDebugger::dump() {
     return;
   }
 
-  // bcc has only one compilation unit
-  // getCompileUnitAtIndex() was gone in llvm 8.0 (https://reviews.llvm.org/D49741)
-#if LLVM_MAJOR_VERSION >= 8
   DWARFCompileUnit *CU = cast<DWARFCompileUnit>(DwarfCtx->getUnitAtIndex(0));
-#else
-  DWARFCompileUnit *CU = DwarfCtx->getCompileUnitAtIndex(0);
-#endif
   if (!CU) {
     errs() << "Debug Error: dwarf context failed to get compile unit\n";
     return;
@@ -202,7 +197,7 @@ void SourceDebugger::dump() {
     uint64_t Size;
     uint8_t *FuncStart = info.start_;
     uint64_t FuncSize = info.size_;
-#if LLVM_MAJOR_VERSION >= 9
+
     auto section = sections_.find(info.section_);
     if (section == sections_.end()) {
       errs() << "Debug Error: no section entry for section " << info.section_
@@ -210,7 +205,7 @@ void SourceDebugger::dump() {
       return;
     }
     unsigned SectionID = get<2>(section->second);
-#endif
+
     ArrayRef<uint8_t> Data(FuncStart, FuncSize);
     uint32_t CurrentSrcLine = 0;
 
@@ -219,12 +214,7 @@ void SourceDebugger::dump() {
     string src_dbg_str;
     llvm::raw_string_ostream os(src_dbg_str);
     for (uint64_t Index = 0; Index < FuncSize; Index += Size) {
-#if LLVM_MAJOR_VERSION >= 10
       S = DisAsm->getInstruction(Inst, Size, Data.slice(Index), Index, nulls());
-#else
-      S = DisAsm->getInstruction(Inst, Size, Data.slice(Index), Index, nulls(),
-                                 nulls());
-#endif
       if (S != MCDisassembler::Success) {
         os << "Debug Error: disassembler failed: " << std::to_string(S) << '\n';
         break;
@@ -232,10 +222,9 @@ void SourceDebugger::dump() {
         DILineInfo LineInfo;
 
         LineTable->getFileLineInfoForAddress(
-#if LLVM_MAJOR_VERSION >= 9
             {(uint64_t)FuncStart + Index, SectionID},
-#else
-            (uint64_t)FuncStart + Index,
+#if LLVM_VERSION_MAJOR >= 20
+            false,
 #endif
             CU->getCompilationDir(),
             DILineInfoSpecifier::FileLineInfoKind::AbsoluteFilePath, LineInfo);
@@ -245,11 +234,7 @@ void SourceDebugger::dump() {
                     os);
         os << format("%4" PRIu64 ":", Index >> 3) << '\t';
         dumpBytes(Data.slice(Index, Size), os);
-#if LLVM_MAJOR_VERSION >= 10
         IP->printInst(&Inst, 0, "", *STI, os);
-#else
-        IP->printInst(&Inst, os, "", *STI);
-#endif
         os << '\n';
       }
     }
