@@ -17,6 +17,7 @@
 
 #include <fcntl.h>
 #include <linux/bpf.h>
+#include <llvm/Config/llvm-config.h>
 #if LLVM_VERSION_MAJOR <= 16
 #include <llvm-c/Transforms/IPO.h>
 #endif
@@ -122,17 +123,10 @@ class MyMemoryManager : public SectionMemoryManager {
       if (!section)
         continue;
 
-#if LLVM_VERSION_MAJOR >= 10
       auto sec_name = section.get()->getName();
       if (!sec_name)
         continue;
-#else
-      llvm::StringRef sec_name_obj;
-      if (!section.get()->getName(sec_name_obj))
-        continue;
 
-      auto sec_name = &sec_name_obj;
-#endif
       info->section_ = sec_name->str();
       info->size_ = ss.second;
     }
@@ -159,11 +153,9 @@ BPFModule::BPFModule(unsigned flags, TableStorage *ts, bool rw_engine_enabled,
   LLVMInitializeBPFTargetMC();
   LLVMInitializeBPFTargetInfo();
   LLVMInitializeBPFAsmPrinter();
-#if LLVM_VERSION_MAJOR >= 6
   LLVMInitializeBPFAsmParser();
   if (flags & DEBUG_SOURCE)
     LLVMInitializeBPFDisassembler();
-#endif
   LLVMLinkInMCJIT(); /* call empty function to force linking of MCJIT */
   if (!ts_) {
     local_ts_ = createSharedTableStorage();
@@ -551,20 +543,17 @@ int BPFModule::finalize() {
   sec_map_def tmp_sections,
       *sections_p;
 
+#if LLVM_VERSION_MAJOR >= 21
+  mod->setTargetTriple(Triple("bpf-pc-linux"));
+#else
   mod->setTargetTriple("bpf-pc-linux");
-#if LLVM_VERSION_MAJOR >= 11
+#endif
 #if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
   mod->setDataLayout("e-m:e-p:64:64-i64:64-i128:128-n32:64-S128");
 #else
   mod->setDataLayout("E-m:e-p:64:64-i64:64-i128:128-n32:64-S128");
 #endif
-#else
-#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
-  mod->setDataLayout("e-m:e-p:64:64-i64:64-n32:64-S128");
-#else
-  mod->setDataLayout("E-m:e-p:64:64-i64:64-n32:64-S128");
-#endif
-#endif
+
   sections_p = rw_engine_enabled_ ? &sections_ : &tmp_sections;
 
   string err;
@@ -573,6 +562,7 @@ int BPFModule::finalize() {
   builder.setMCJITMemoryManager(
       ebpf::make_unique<MyMemoryManager>(sections_p, &*prog_func_info_));
   builder.setMArch("bpf");
+  builder.setMCPU("v1");
 #if LLVM_VERSION_MAJOR <= 11
   builder.setUseOrcMCJITReplacement(false);
 #endif
