@@ -179,13 +179,7 @@ static int CreateFromArgs(clang::CompilerInvocation &invocation,
                           const llvm::opt::ArgStringList &ccargs,
                           clang::DiagnosticsEngine &diags)
 {
-#if LLVM_VERSION_MAJOR >= 10
   return clang::CompilerInvocation::CreateFromArgs(invocation, ccargs, diags);
-#else
-  return clang::CompilerInvocation::CreateFromArgs(
-              invocation, const_cast<const char **>(ccargs.data()),
-              const_cast<const char **>(ccargs.data()) + ccargs.size(), diags);
-#endif
 }
 
 }
@@ -266,6 +260,7 @@ int ClangLoader::parse(
                                    "-Wno-address-of-packed-member",
                                    "-Wno-unknown-warning-option",
 #if defined(__x86_64__) || defined(__i386__)
+                                   "-Wno-duplicate-decl-specifier",
                                    "-fcf-protection",
 #endif
                                    "-fno-color-diagnostics",
@@ -287,13 +282,10 @@ int ClangLoader::parse(
   vector<string> kflags;
   if (kbuild_helper.get_flags(un.machine, &kflags))
     return -1;
-#if LLVM_VERSION_MAJOR >= 9
+
   flags_cstr.push_back("-g");
   flags_cstr.push_back("-gdwarf-4");
-#else
-  if (flags_ & DEBUG_SOURCE)
-    flags_cstr.push_back("-g");
-#endif
+
   for (auto it = kflags.begin(); it != kflags.end(); ++it)
     flags_cstr.push_back(it->c_str());
 
@@ -415,10 +407,8 @@ int ClangLoader::do_compile(
   string target_triple = get_clang_target();
   driver::Driver drv("", target_triple, diags);
 
-#if LLVM_VERSION_MAJOR >= 4
   if (target_triple == "x86_64-unknown-linux-gnu" || target_triple == "aarch64-unknown-linux-gnu")
     flags_cstr.push_back("-fno-jump-tables");
-#endif
 
   drv.setTitle("bcc-clang-driver");
   drv.setCheckInputsExist(false);
@@ -466,7 +456,11 @@ int ClangLoader::do_compile(
   }
   invocation0.getFrontendOpts().DisableFree = false;
 
+#if LLVM_VERSION_MAJOR >= 20
+  compiler0.createDiagnostics(*llvm::vfs::getRealFileSystem(), new IgnoringDiagConsumer());
+#else
   compiler0.createDiagnostics(new IgnoringDiagConsumer());
+#endif
 
   // capture the rewritten c file
   string out_str;
@@ -485,7 +479,11 @@ int ClangLoader::do_compile(
   add_main_input(invocation1, main_path, &*out_buf);
   invocation1.getFrontendOpts().DisableFree = false;
 
+#if LLVM_VERSION_MAJOR >= 20
+  compiler1.createDiagnostics(*llvm::vfs::getRealFileSystem());
+#else
   compiler1.createDiagnostics();
+#endif
 
   // capture the rewritten c file
   string out_str1;
@@ -511,7 +509,11 @@ int ClangLoader::do_compile(
   invocation2.getCodeGenOpts().setInlining(CodeGenOptions::NormalInlining);
   // suppress warnings in the 2nd pass, but bail out on errors (our fault)
   invocation2.getDiagnosticOpts().IgnoreWarnings = true;
+#if LLVM_VERSION_MAJOR >= 20
+  compiler2.createDiagnostics(*llvm::vfs::getRealFileSystem());
+#else
   compiler2.createDiagnostics();
+#endif
 
   EmitLLVMOnlyAction ir_act(&*ctx_);
   if (!compiler2.ExecuteAction(ir_act))
