@@ -197,6 +197,10 @@ b = BPF(text=bpf_text)
 b.attach_kprobe(event="vfs_read", fn_name="trace_read_entry")
 b.attach_kprobe(event="vfs_write", fn_name="trace_write_entry")
 
+# check whether hash table batch ops is supported
+htab_batch_ops = True if BPF.kernel_struct_has_field(b'bpf_map_ops',
+        b'map_lookup_and_delete_batch') == 1 else False
+
 DNAME_INLINE_LEN = 32  # linux/dcache.h
 
 print('Tracing... Output every %d secs. Hit Ctrl-C to end' % interval)
@@ -235,7 +239,8 @@ while 1:
     writes = {}
     reads_Kb = {}
     writes_Kb = {}
-    for k, v in reversed(sorted(counts.items(),
+    for k, v in reversed(sorted(counts.items_lookup_and_delete_batch()
+                                if htab_batch_ops else counts.items(),
                                 key=sort_fn)):
         # If it's the first time we see this inode
         if k.inode_id not in reads:
@@ -259,7 +264,8 @@ while 1:
         if line >= maxrows:
             break
 
-    counts.clear()
+    if not htab_batch_ops:
+        counts.clear()
 
     countdown -= 1
     if exiting or countdown == 0:
