@@ -21,6 +21,7 @@ import argparse
 examples = """examples:
     ./statsnoop           # trace all stat() syscalls
     ./statsnoop -t        # include timestamps
+    ./statsnoop -s        # include syscall name
     ./statsnoop -x        # only show failed stats
     ./statsnoop -p 181    # only trace PID 181
 """
@@ -30,6 +31,8 @@ parser = argparse.ArgumentParser(
     epilog=examples)
 parser.add_argument("-t", "--timestamp", action="store_true",
     help="include timestamp on output")
+parser.add_argument("-s", "--sysname", action="store_true",
+    help="include syscall name on output")
 parser.add_argument("-x", "--failed", action="store_true",
     help="only show failed stats")
 parser.add_argument("-p", "--pid",
@@ -45,8 +48,19 @@ bpf_text = """
 #include <uapi/linux/limits.h>
 #include <linux/sched.h>
 
+enum sys_type {
+    SYS_STAT = 1,
+    SYS_STATX,
+    SYS_STATFS,
+    SYS_NEWSTAT,
+    SYS_NEWLSTAT,
+    SYS_FSTATAT64,
+    SYS_NEWFSTATAT,
+};
+
 struct val_t {
     const char *fname;
+    enum sys_type type;
 };
 
 struct data_t {
@@ -55,12 +69,14 @@ struct data_t {
     int ret;
     char comm[TASK_COMM_LEN];
     char fname[NAME_MAX];
+    u32 type; /* enum sys_type */
 };
 
 BPF_HASH(infotmp, u32, struct val_t);
 BPF_PERF_OUTPUT(events);
 
-static int trace_entry(struct pt_regs *ctx, const char __user *filename)
+static int trace_entry(struct pt_regs *ctx, enum sys_type type,
+                       const char __user *filename)
 {
     struct val_t val = {};
     u64 pid_tgid = bpf_get_current_pid_tgid();
@@ -69,19 +85,45 @@ static int trace_entry(struct pt_regs *ctx, const char __user *filename)
 
     FILTER
     val.fname = filename;
+    val.type = type;
     infotmp.update(&tid, &val);
 
     return 0;
 };
 
-int stat_entry(struct pt_regs *ctx, const char __user *filename)
+int syscall__stat_entry(struct pt_regs *ctx, const char __user *filename)
 {
-    return trace_entry(ctx, filename);
+    return trace_entry(ctx, SYS_STAT, filename);
 }
 
-int statx_entry(struct pt_regs *ctx, int dfd, const char __user *filename)
+int syscall__statfs_entry(struct pt_regs *ctx, const char __user *filename)
 {
-    return trace_entry(ctx, filename);
+    return trace_entry(ctx, SYS_STATFS, filename);
+}
+
+int syscall__newstat_entry(struct pt_regs *ctx, const char __user *filename)
+{
+    return trace_entry(ctx, SYS_NEWSTAT, filename);
+}
+
+int syscall__newlstat_entry(struct pt_regs *ctx, const char __user *filename)
+{
+    return trace_entry(ctx, SYS_NEWLSTAT, filename);
+}
+
+int syscall__statx_entry(struct pt_regs *ctx, int dfd, const char __user *filename)
+{
+    return trace_entry(ctx, SYS_STATX, filename);
+}
+
+int syscall__fstatat64_entry(struct pt_regs *ctx, int dfd, const char __user *filename)
+{
+    return trace_entry(ctx, SYS_FSTATAT64, filename);
+}
+
+int syscall__newfstatat_entry(struct pt_regs *ctx, int dfd, const char __user *filename)
+{
+    return trace_entry(ctx, SYS_NEWFSTATAT, filename);
 }
 
 int trace_return(struct pt_regs *ctx)
@@ -98,6 +140,7 @@ int trace_return(struct pt_regs *ctx)
 
     struct data_t data = {.pid = pid_tgid >> 32};
     bpf_probe_read_user(&data.fname, sizeof(data.fname), (void *)valp->fname);
+    data.type = valp->type;
     bpf_get_current_comm(&data.comm, sizeof(data.comm));
     data.ts_ns = bpf_ktime_get_ns();
     data.ret = PT_REGS_RC(ctx);
@@ -128,10 +171,7 @@ b = BPF(text=bpf_text)
 def try_attach_syscall_probes(syscall):
     syscall_fnname = b.get_syscall_fnname(syscall)
     if BPF.ksymname(syscall_fnname) != -1:
-        if syscall == "statx":
-            b.attach_kprobe(event=syscall_fnname, fn_name="statx_entry")
-        else:
-            b.attach_kprobe(event=syscall_fnname, fn_name="stat_entry")
+        b.attach_kprobe(event=syscall_fnname, fn_name="syscall__%s_entry" % syscall)
         b.attach_kretprobe(event=syscall_fnname, fn_name="trace_return")
 
 try_attach_syscall_probes("stat")
@@ -139,6 +179,20 @@ try_attach_syscall_probes("statx")
 try_attach_syscall_probes("statfs")
 try_attach_syscall_probes("newstat")
 try_attach_syscall_probes("newlstat")
+try_attach_syscall_probes("fstatat64")
+try_attach_syscall_probes("newfstatat")
+
+# See enum sys_type.
+sys_names = [
+    "N/A",
+    "stat",
+    "statx",
+    "statfs",
+    "newstat",
+    "newlstat",
+    "fstatat64",
+    "newfstatat",
+]
 
 start_ts = 0
 prev_ts = 0
@@ -147,7 +201,10 @@ delta = 0
 # header
 if args.timestamp:
     print("%-14s" % ("TIME(s)"), end="")
-print("%-7s %-16s %4s %3s %s" % ("PID", "COMM", "FD", "ERR", "PATH"))
+print("%-7s %-16s %4s %3s" % ("PID", "COMM", "FD", "ERR"), end="")
+if args.sysname:
+    print(" %-12s" % "SYSCALL", end="")
+print(" %s" % "PATH")
 
 # process event
 def print_event(cpu, data, size):
@@ -173,9 +230,13 @@ def print_event(cpu, data, size):
     if args.timestamp:
         print("%-14.9f" % (float(event.ts_ns - start_ts) / 1000000000), end="")
 
-    print("%-7d %-16s %4d %3d %s" % (event.pid,
-        event.comm.decode('utf-8', 'replace'), fd_s, err,
-        event.fname.decode('utf-8', 'replace')))
+    print("%-7d %-16s %4d %3d" % (event.pid,
+        event.comm.decode('utf-8', 'replace'), fd_s, err), end="")
+
+    if args.sysname:
+        print(" %-12s" % sys_names[event.type], end="")
+
+    print(" %s" % event.fname.decode('utf-8', 'replace'))
 
 # loop with callback to print_event
 b["events"].open_perf_buffer(print_event, page_cnt=64)
